SPEC=	$(firstword $(wildcard $(PRJ_BASE)/*.spec))
VER=	$(shell rpmspec -q --srpm --qf '%{version}' $(SPEC))

ifeq (,$(findstring rpmbuild/BUILD,$(CURDIR)))

NAME=	$(shell rpmspec -q --srpm --qf '%{name}' $(SPEC))
REL=	$(shell rpmspec -q --srpm --qf '%{release}' $(SPEC))
ARCH=	$(shell rpmspec -q --srpm --qf '%{arch}' $(SPEC))
RPMS=	$(addprefix $(shell rpmbuild --eval %_rpmdir)/$(ARCH)/,\
	$(addsuffix .rpm,$(shell rpmspec -q $(SPEC))))
RPM=	$(firstword $(RPMS))
TAG?=	V$(subst .,,$(VER))

REL_NOTE=		$(firstword $(wildcard *relnote.txt))

INSTALL_DIR=		$(shell pwd)/dnf_tree
INSTALL_RPMS_DIR=	RPMS
INSTALL_REL_DIR=	relnote

SERV=			ulmdnf
SERV_USER=		ulm
SERV_RC_DIR=	/pub/www/doc/RC/ulm
SERV_V120_DIR=	/pub/www/doc/release/V120/ulm/
SERV_USR_DIR=	/pub/home/ulm
DNF_REPO=		TRUE

REL_VER=		none

.PHONY:	rpm rpmclean release create_rel_br

rpm:	$(RPM)

$(RPM):	$(SPEC)
#	rpmbuild -bb --sign $<
	rpmbuild -bb $<
	rpm --addsign $(RPMS)

release: $(RPM)
	mkdir -p $(INSTALL_DIR)/$(INSTALL_RPMS_DIR)/$(ARCH)
	mkdir -p $(INSTALL_DIR)/$(INSTALL_REL_DIR)
	ln -sf $(RPMS) $(INSTALL_DIR)/$(INSTALL_RPMS_DIR)/$(ARCH)
	ln -sf ../../$(REL_NOTE) $(INSTALL_DIR)/$(INSTALL_REL_DIR)
	rsync -vaL $(INSTALL_DIR)/$(INSTALL_REL_DIR) $(SERV_USER)@$(SERV):$(SERV_USR_DIR)/
	if [ "$(DNF_REPO)" = "V120" ]; then \
		rsync -vaL $(INSTALL_DIR)/$(INSTALL_RPMS_DIR) $(SERV_USER)@$(SERV):$(SERV_V120_DIR); \
		ssh $(SERV_USER)@$(SERV) /usr/bin/updaterepo -V V120; \
	elif [ "$(DNF_REPO)" = "RC" ]; then \
		rsync -vaL $(INSTALL_DIR)/$(INSTALL_RPMS_DIR) $(SERV_USER)@$(SERV):$(SERV_RC_DIR); \
		ssh $(SERV_USER)@$(SERV) /usr/bin/updaterepo -R; \
	elif [ "$(DNF_REPO)" = "TRUE" ]; then \
		rsync -vaL $(INSTALL_DIR)/$(INSTALL_RPMS_DIR) $(SERV_USER)@$(SERV):$(SERV_RC_DIR); \
		ssh $(SERV_USER)@$(SERV) /usr/bin/updaterepo -R; \
	else \
		echo "Illegal Repo: $(DNF_REPO)"; \
	fi;

rpmclean:
	$(RM) $(RPMS)

create_rel_br:
	$(shell pwd)/make/script/create_rel_br.sh -v $(REL_VER) -b $(shell git rev-parse --abbrev-ref HEAD) -x

endif
